#include "forms.h"
#include "confirm.h"
#include "FvwmCDE.h"
#include "interaction.h"
#include "minimized.h"
#include "defs.h"
#include "interface.h"
#include "sundry.h"
#include "panel.h"

#ifdef USE_HINTS
#include "tooltips.h"
#endif

#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "icons/light_on_pixmap.xpm"
#include "icons/light_off_pixmap.xpm"

/* global variables (ack!) */
FD_FvwmCDE *fd_FvwmCDE;
FD_FvwmCDE_minimized *fd_FvwmCDE_minimized;
FD_panel *Panel[10];
FD_confirm *fd_confirm;
char MailFile[_POSIX_PATH_MAX];
Display *disp;
GC canvasGC;
char CmdList[NWIDGETS][CMDLEN];

int ledState;			/* whether the LED is bright (1) or dim (0) */
int ledCounter;			/* count the apps we've launched */
int disable_all = 0;		/* if the Confirm window is up.. */

/* catch those blasted bugs.. */
void
IGoofed (int signal)
{
  fprintf (stderr, "FvwmCDE: Ack, received SIGSEGV\n");
  fflush (stderr);
  exit (1);
}

/* main program -- GUI initialization and stuff */
int
main (int argc, char *argv[])
{
  int screen, xofs, yofs, w, h, scr_w, scr_h, count;
  FL_IOPT flopt;
  FL_OBJECT *ret;
  char *ch;

  /* for Fvwm messaging */
  fd_set in_fdset;
  struct timeval in_timer;
  unsigned long header[3], *body;

  /* setup the LED state stuff */
  ledState = ledCounter = 0;

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: starting up\n");
#else
  /* we can use GDB to post-mortem core dumps */
  signal (SIGSEGV, IGoofed);
#endif

  /* get the mailfile */
  memset (MailFile, 0, _POSIX_PATH_MAX);
  ch = getenv ("MAIL");
  if (ch == NULL)		/* must be using csh */
    {				/* try to construct from $USER */
      ch = getenv ("USER");
      if (ch != NULL)
	sprintf (MailFile, "%s/%s", MAILDIR, ch);
      else
	fprintf (stderr, "FvwmCDE: couldn't determine mail file\n");
    }
  else
    {				/* tcsh and bash define $MAIL */
      strcpy (MailFile, ch);
    }

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: initializing X connection.. ");
#endif

  flopt.borderWidth = (-2);
  fl_set_defaults (FL_PDBorderWidth, &flopt);

  fl_initialize (&argc, argv, 0, 0, 0);
  fl_set_goodies_font (FL_BOLD_STYLE, FL_NORMAL_STYLE);

  /* make sure we use "normal" width fonts */
  fl_set_font_name (FL_NORMAL_STYLE,
		    "-*-helvetica-medium-r-normal-*-*-?-100-100-p-*-*-*");
  fl_set_font_name (FL_BOLD_STYLE,
		    "-*-helvetica-bold-r-normal-*-*-?-100-100-p-*-*-*");
  fl_set_font_name (FL_ITALIC_STYLE,
		    "-*-helvetica-medium-o-normal-*-*-?-100-100-p-*-*-*");
  fl_set_font_name (FL_BOLDITALIC_STYLE,
		    "-*-helvetica-bold-o-normal-*-*-?-100-100-p-*-*-*");

  /* this is just for the calendar; unfortunately, some X servers (e.g. */
  /* the one in SGI IRIX 5.3) don't have a 72-dpi Lucida font by default */
#ifndef NO_LUCIDA
  fl_set_font_name (FL_FIXED_STYLE,
		    "-*-lucida-medium-r-normal-*-*-140-75-75-p-*-*-*");
#else
  fl_set_font_name (FL_FIXED_STYLE,
		    "-*-helvetica-medium-r-normal-*-*-?-75-75-p-*-*-*");
#endif

#ifdef USE_HINTS
  tooltips_initialize ();
#endif

  fd_FvwmCDE = create_form_FvwmCDE ();
  fd_FvwmCDE_minimized = create_form_FvwmCDE_minimized ();
  fd_confirm = create_form_confirm ();
#ifdef USE_DOUBLEBUFFER
  fl_set_form_dblbuffer (fd_FvwmCDE->FvwmCDE, 1);
#endif

  /* initialize the pop-out panels */
  for (count = 0; count < 10; count++)
    Panel[count] = create_form_panel (MAX_ENTRIES);

  /* set sizing and location */
  disp = fl_get_display ();
  screen = DefaultScreen (disp);
  scr_w = DisplayWidth (disp, screen);
  scr_h = DisplayHeight (disp, screen);

  w = fd_FvwmCDE->FvwmCDE->w;
  h = fd_FvwmCDE->FvwmCDE->h;
  xofs = (scr_w - w) / 2;
  yofs = (scr_h + 2) - h;
  fl_set_form_geometry (fd_FvwmCDE->FvwmCDE, xofs, yofs,
			fd_FvwmCDE->FvwmCDE->w, fd_FvwmCDE->FvwmCDE->h);
  fl_set_form_geometry (fd_FvwmCDE_minimized->FvwmCDE_minimized, 5, -5,
			fd_FvwmCDE_minimized->FvwmCDE_minimized->w,
			fd_FvwmCDE_minimized->FvwmCDE_minimized->h);

  /* set up the clock_canvas */
  fl_add_canvas_handler (fd_FvwmCDE->clock_canvas, Expose,
			 canvas_expose, fd_FvwmCDE);
  fl_add_canvas_handler (fd_FvwmCDE->clock_canvas, ButtonPress,
			 canvas_click, fd_FvwmCDE);
  canvasGC = XCreateGC (fl_get_display (),
			fl_state[fl_vmode].trailblazer, 0, 0);

#ifndef NODEBUG
  fprintf (stderr, "done\n");
#endif

  /* find out if we're connected to Fvwm */
  if (argc < 6)
    {
      fprintf (stderr, "Error: not spawned by Fvwm\n");
      exit (1);
    }
  else
    {
      signal (SIGPIPE, DeadPipe);
      FvwmPipe[0] = atoi (argv[1]);
      FvwmPipe[1] = atoi (argv[2]);

      /* nullify the bindings.. */
      for (count = 0; count < NWIDGETS; count++)
	memset (CmdList[count], 0, CMDLEN);

      /* and read our configuration */
      ParseConfig ();

      /* Go to first desk (as that's what our button says!) */
      SendInfo (FvwmPipe, "Desk 0 0", 0);
      if (strlen (dt1back) > 0)
       system (dt1back);
      FvwmCDE_desktop = 1;       

      /* and listen for ConfigureWindow packets */
      SendInfo (FvwmPipe, "Send_WindowList", 0);
    }

  /* do some basic color setup and stuff.. */
  /* this must be AFTER the configuration is parsed */
  (void) fl_mapcolor (CUSTOM_COLOR1, BackRed, BackGreen, BackBlue);
  (void) fl_mapcolor (CUSTOM_COLOR2, ButtonRed, ButtonGreen, ButtonBlue);
  (void) fl_mapcolor (CUSTOM_COLOR3, 0xbd, 0xbd, 0xbd);
  (void) fl_mapcolor (CUSTOM_COLOR4, 0x93, 0x97, 0xa5);

  /* show the first form */
  fl_show_form (fd_FvwmCDE->FvwmCDE, FL_PLACE_GEOMETRY, FL_TRANSIENT,
		"FvwmCDE");
  fl_set_button (fd_FvwmCDE->one_button, 1);
  fl_set_button (fd_FvwmCDE->two_button, 0);
  fl_set_button (fd_FvwmCDE->three_button, 0);
  fl_set_button (fd_FvwmCDE->four_button, 0);

  /* do interaction, listen for messages from Big Brother Fvwm */
  ret = fl_check_forms ();

  /* infinite loop.. */
  while (1)
    {
      /* here's where we handle the LED */
      if (ledCounter > 0 && !(ledCounter % 20))
	{
	  /* time to blink the LED */
	  fl_free_pixmap_pixmap (fd_FvwmCDE->light_pixmap);
	  if (ledState == 0)
	    {
	      fl_set_pixmap_data (fd_FvwmCDE->light_pixmap,
				  light_on_pixmap_xpm);
	      ledState = 1;
	    }
	  else
	    {
	      fl_set_pixmap_data (fd_FvwmCDE->light_pixmap,
				  light_off_pixmap_xpm);
	      ledState = 0;
	    }
	}

      if (ledCounter == 0 && ledState == 1)
	{
	  fl_free_pixmap_pixmap (fd_FvwmCDE->light_pixmap);
	  fl_set_pixmap_data (fd_FvwmCDE->light_pixmap,
			      light_off_pixmap_xpm);
	  ledState = 0;
	}

      update_calendar ();

      FD_ZERO (&in_fdset);
      FD_SET (FvwmPipe[1], &in_fdset);
      in_timer.tv_sec = 0;
      in_timer.tv_usec = 100000;	/* this ought to be fast enough */
      select (FvwmPipe[1] + 1, &in_fdset, NULL, NULL, &in_timer);

      /* decrement the ledCounter by 10 jiffies */
      ledCounter -= 10;;	/* 10ms == 1 jiffy */

      if (FD_ISSET (FvwmPipe[1], &in_fdset))
	{
	  if ((count = ReadFvwmPacket (FvwmPipe[1], header, &body)) > 0)
	    {
	      process_message (header[1], body);
	      free (body);
	    }
	}

      ret = fl_check_forms ();
      if (ret == fd_FvwmCDE_minimized->iconic_button)
	{
	  fl_hide_form (fd_FvwmCDE_minimized->FvwmCDE_minimized);
	  fl_show_form (fd_FvwmCDE->FvwmCDE, FL_PLACE_GEOMETRY,
			FL_TRANSIENT, "FvwmCDE");
	}

      if (ret == fd_confirm->ok_button)
	SendInfoCheck (FvwmPipe, CmdList[EXIT], 0);

      if (ret == fd_confirm->cancel_button)
	{
	  fl_hide_form (fd_confirm->confirm);
	  fl_activate_form (fd_FvwmCDE->FvwmCDE);
	  disable_all = 0;
	}

      if (!disable_all)
	{
	  check_buttons (fd_FvwmCDE, ret);
	  for (count = 0; count < 10; count++)
	    check_panels (Panel[count], ret);
	}
    }
  return 0;
}
