#include "forms.h"
#include "panel.h"
#include "icons/default.xpm"
#include "icons/little_pixmap.xpm"
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

FD_panel *
create_form_panel (int num_entries)
{
  int i, y, size;
  FL_OBJECT *obj;
  FD_panel *fdui = (FD_panel *) fl_calloc (1, sizeof (*fdui));

  if (num_entries > MAX_ENTRIES)
    {
      fprintf (stderr, "Too many entries: %d maximum\n", MAX_ENTRIES);
      num_entries = MAX_ENTRIES;
    }

  size = (num_entries * 55) + 20;
  if (num_entries > 1)
    size += ((num_entries - 1) * 5);

  fdui->panel = fl_bgn_form (FL_NO_BOX, 240, size);
  fdui->panelbox = obj =
    fl_add_box (FL_UP_BOX, 0, 0, 240, size, "");
  fl_set_object_resize (obj, FL_RESIZE_NONE);
  fl_set_object_color (obj, CUSTOM_COLOR1, CUSTOM_COLOR1);
  fl_set_object_bw (obj, -1);

  for (i = 0; i < num_entries; i++)
    {
      y = (i * 60) + 10;
      fdui->button_pixmap[i] = obj =
	fl_add_pixmapbutton (FL_NORMAL_BUTTON, 10, y, 55, 55, "");
      fl_set_pixmapbutton_data (obj, default_xpm);
      fl_set_object_boxtype (obj, FL_DOWN_BOX);
      fl_set_object_lsize (obj, FL_NORMAL_SIZE);
      fl_set_object_lalign (obj, FL_ALIGN_RIGHT);
      fl_set_object_lstyle (obj, FL_BOLD_STYLE + FL_SHADOW_STYLE);
      fl_set_object_color (obj, CUSTOM_COLOR1, CUSTOM_COLOR2);
      fl_set_object_resize (obj, FL_RESIZE_NONE);
      fl_set_object_bw (obj, -1);

      fdui->button_label[i] = obj =
	fl_add_button (FL_NORMAL_BUTTON, 75, y, 150, 55, "");
      fl_set_object_boxtype (obj, FL_FLAT_BOX);
      fl_set_object_lcol (obj, FL_WHITE);
      fl_set_object_lsize (obj, FL_NORMAL_SIZE);
      fl_set_object_lalign (obj, FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
      fl_set_object_lstyle (obj, FL_BOLD_STYLE + FL_SHADOW_STYLE);
      fl_set_object_color (obj, CUSTOM_COLOR1, CUSTOM_COLOR2);
      fl_set_object_resize (obj, FL_RESIZE_NONE);

      memset (fdui->button_action[i], 0, CMDLEN);
      strcpy (fdui->label, "Panel");
    }

  fl_end_form ();

  fdui->panel->fdui = fdui;

  return fdui;
}

/* adds entries to a specified panel
 * each entry line is of the form (label _must_ be quoted!):
 *  *FvwmCDE*Panel0*binding "label%pixmap.xpm%" action
 */
void
add_to_panel (FD_panel * panel, char *line)
{
  char *ch, *tmp;
  char label[CMDLEN], action[CMDLEN], pixmap[CMDLEN], path[CMDLEN], localpath[_POSIX_PATH_MAX * 8],
    pixmapfile[_POSIX_PATH_MAX];
  struct stat stbuf;
  int i, n, found = 0, len;

  if (line == NULL || *line == '\0')
    return;

  for (i = 0; i < MAX_ENTRIES && !found; i++)
    {
      if (strlen (panel->button_action[i]) == 0)
	{
	  found = 1;
	  n = i;
	}
    }

  if (!found)			/* too many entries */
    return;

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: [add_to_panel] got line:\n%s", line);
#endif

  /* add a little pointy pixmap to the appropriate button */
  if (panel == Panel[0])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[0]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[0],
				little_pixmap_xpm);
    }

  if (panel == Panel[1])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[1]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[1],
				little_pixmap_xpm);
    }

  if (panel == Panel[2])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[2]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[2],
				little_pixmap_xpm);
    }

  if (panel == Panel[3])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[3]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[3],
				little_pixmap_xpm);
    }

  if (panel == Panel[4])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[4]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[4],
				little_pixmap_xpm);
    }

  if (panel == Panel[5])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[5]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[5],
				little_pixmap_xpm);
    }

  if (panel == Panel[6])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[6]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[6],
				little_pixmap_xpm);
    }

  if (panel == Panel[7])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[7]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[7],
				little_pixmap_xpm);
    }

  if (panel == Panel[8])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[8]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[8],
				little_pixmap_xpm);
    }

  if (panel == Panel[9])
    {
      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->panelbutton[9]);
      fl_set_pixmapbutton_data (fd_FvwmCDE->panelbutton[9],
				little_pixmap_xpm);
    }


  ch = line;
  while (*ch != '\0' && isspace (*ch))
    ch++;

  /* label */
  if ((*ch == '"') && (*(ch + 1) != '\0'))
    {
      strcpy (label, ch + 1);
      tmp = strchr (label, '"');	/* terminating quote */
      if (tmp == NULL)		/* unterminated */
	{
	  fprintf (stderr, "FvwmCDE: [add_to_panel] unterminated label:\n"
		   "%s", line);
	  return;
	}
      *tmp = '\0';		/* chop it off */
      if (*(tmp + 1) != '\0')	/* action */
	{
	  ch = tmp + 1;
	  while (*ch != '\0' && isspace (*ch))
	    ch++;
	}
    }
  strcpy (action, ch);

  memset (pixmap, 0, CMDLEN);
  memset (pixmapfile, 0, CMDLEN);

  ch = strchr (label, '%');	/* pixmap filename delimiter */
  if (ch != NULL)
    {
      *ch = '\0';
      if (*(ch + 1) != '\0')
	{
	  strcpy (pixmap, ch + 1);
	  tmp = strchr (ch + 1, '%');
	  if (tmp != NULL && *(tmp + 1) != '\0')
	    memmove (ch, tmp, strlen (tmp) + 1);
	  tmp = strchr (pixmap, '%');
	  if (tmp != NULL)
	    *tmp = '\0';
	}
    }

  fl_set_object_label (panel->button_label[n], label);
  strcpy (panel->button_action[n], action);

  /* look for the pixmap */
#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: [add_to_panel] Parsed pixmap %s\n\n",
	   pixmap);
#endif
  if (strlen (pixmap) > 0)
    {
      found = 0;
      strcpy (localpath, PixmapPath);
      ch = strtok (localpath, ":");

      while (!found && ch != NULL)
	{
	  memset (path, _POSIX_PATH_MAX, 0);
	  sprintf (path, "%s/%s", ch, pixmap);

	  if (stat (path, &stbuf) == 0)
	    {
	      strcpy (pixmapfile, path);
	      found = 1;
	    }
	  else
	    {
	      ch = strtok (NULL, ":");
	    }
	}

      if (!found)		/* one last try.. */
	{
#ifndef NODEBUG	
	  sprintf (path, "%s/%s", PixmapPath, pixmap);
	  fprintf (stderr, "Final trial pixmapfile: [%s]\n", path);
	  fflush (stderr);
#endif
	  if (stat (path, &stbuf) == 0)
	    {
	      strcpy (pixmapfile, path);
	      found = 1;
	    }
	}

      if (found && strlen (pixmapfile) > 0)
	{
	  fl_free_pixmapbutton_pixmap (panel->button_pixmap[n]);
	  fl_set_pixmapbutton_file (panel->button_pixmap[n], pixmapfile);
	}
    }

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: [add_to_panel]\nLabel: %s\nPixmap: %s(%s)\n"
	   "Action: %s\n",
	   label, pixmap, pixmapfile, action);
  fflush (stderr);
#endif

  return;
}

/* free unused panel entries and resize panel */
/* if the panel has no entries, deactivate its button */
void
trim_panel (FD_panel * panel)
{
  int i, n = 0, size, y;

  fl_freeze_form (panel->panel);
  for (i = 0; i < MAX_ENTRIES; i++)
    if (strlen (panel->button_action[i]) > 0)
      n = i + 1;
    else
      {
	/* no bound action -- free it */
	fl_free_object (panel->button_pixmap[i]);
	fl_free_object (panel->button_label[i]);
      }

  if (n > MAX_ENTRIES)
    n = MAX_ENTRIES;

  if (n < 1)
    {
      fl_deactivate_form (panel->panel);

      if (panel == Panel[0])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[0]);

      if (panel == Panel[1])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[1]);

      if (panel == Panel[2])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[2]);

      if (panel == Panel[3])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[3]);

      if (panel == Panel[4])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[4]);

      if (panel == Panel[5])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[5]);

      if (panel == Panel[6])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[6]);

      if (panel == Panel[7])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[7]);

      if (panel == Panel[8])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[8]);

      if (panel == Panel[9])
	fl_deactivate_object (fd_FvwmCDE->panelbutton[9]);
    }
  else
    {
      size = (n * 55) + ((n - 1) * 5) + 20;

      fl_set_form_size (panel->panel, 240, size);
      fl_set_object_size (panel->panelbox, 240, size);
      fl_set_object_position (panel->panelbox, 0, 0);

      for (i = 0; i < n; i++)
	{
	  y = (i * 60) + 10;
	  fl_set_object_position (panel->button_pixmap[i], 10, y);
	  fl_set_object_position (panel->button_label[i], 75, y);
	}
      fl_unfreeze_form (panel->panel);
    }
}
