/* sundry stuff.. */
#include "forms.h"
#include "defs.h"
#include "sundry.h"
#include "interface.h"
#include "panel.h"
#include "icons/mail_pixmap.xpm"
#include "icons/nomail_pixmap.xpm"

#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

/* for bubble help -- not everyone might have this.. */
#ifdef USE_HINTS
#include "tooltips.h"
#endif

/* global variable (sigh..) */
int CanvasW = 0;
char PixmapPath[_POSIX_PATH_MAX * 8];

/* the colors of the wind.. */
int BackRed, BackGreen, BackBlue;
int ButtonRed, ButtonGreen, ButtonBlue;

/* the background specs and RootWindow command */
char dt1back[_POSIX_PATH_MAX], dt2back[_POSIX_PATH_MAX];
char dt3back[_POSIX_PATH_MAX], dt4back[_POSIX_PATH_MAX];

/* pre-computed values for sin() * 1000, to avoid using floats */
/* I palmed this from Rob Nation's rclock.c */
static const short Sin[60] =
{
  0,
  105, 208, 309, 407, 500, 588, 669,
  743, 809, 866, 914, 951, 978, 995,
  1000,
  995, 978, 951, 914, 866, 809, 743,
  669, 588, 500, 407, 309, 208, 105,
  0,
  -105, -208, -309, -407, -500, -588, -669,
  -743, -809, -866, -914, -951, -978, -995,
  -1000,
  -995, -978, -951, -914, -866, -809, -743,
  -669, -588, -500, -407, -309, -208, -105
};

#define XPOS(i,val) (xc + (w  * Sin[i%60] * (val) + 100000) / 200000)
#define YPOS(i,val) (yc - (h * Sin[(i+15)%60] * (val) + 100000) / 200000)


/* clean up a string -- just for tooltips (for now..) */
char *
cleantip (char *s)
{
  static char cleaned_tip[_POSIX_PATH_MAX];
  char *ch;

  memset (cleaned_tip, 0, _POSIX_PATH_MAX);
  while (*s != '\0' && (isspace (*s) || *s == '"'))
    s++;

  strcpy (cleaned_tip, s);
  ch = strrchr (cleaned_tip, '"');
  if (ch != NULL)
    *ch = '\0';

  ch = strrchr (cleaned_tip, '\n');
  if (ch != NULL)
    *ch = '\0';

  return (cleaned_tip);
}

/* program initialization -- based on ParseConfig from FvwmWinList.c */
void
ParseConfig (void)
{
  char *tline, *ch, *tmp;
  int l, red, green, blue;
  tline = (char *) 1;		/* just to skip the check below */

  /* we put the PixmapPath here */
  memset (PixmapPath, 0, _POSIX_PATH_MAX * 8);

  /* root window settings and command */
  memset (dt1back, 0, _POSIX_PATH_MAX);
  memset (dt2back, 0, _POSIX_PATH_MAX);
  memset (dt3back, 0, _POSIX_PATH_MAX);
  memset (dt4back, 0, _POSIX_PATH_MAX);

  /* default background colors */
  BackRed = 0xae;
  BackGreen = 0xb2;
  BackBlue = 0xc3;
  ButtonRed = 0x71;
  ButtonGreen = 0x8b;
  ButtonBlue = 0xa5;

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: starting to parse configuration\n");
#endif

  while (tline != (char *) 0)
    {
      GetConfigLine (FvwmPipe, &tline);

      if (tline != NULL && strlen (tline) > 9)
	{
	  /* check for PixmapPath */
	  if (!strncasecmp (tline, "PixmapPath", 10))
	    {
	      ch = tline + 10;
	      while (*ch != '\0' && isspace (*ch))
		ch++;
	      strcpy (PixmapPath, ch);

	      /* remove whitespace and doubled colons in PixmapPath */
	      ch = PixmapPath;
	      while (*ch != '\0')
		{
		  l = strlen (ch);

		  if (isspace (*ch))
		    memmove (ch, ch + 1, l);

		  if (*ch == ':' && *(ch + 1) == ':')
		    memmove (ch, ch + 1, l);

		  ch++;
		}
	    }

#ifdef USE_HINTS
	  /* do we want to disable/enable bubble help? */
	  if (!strncasecmp (tline, "*FvwmCDE*EnableHints", 20))
	    tooltips_resume ();

	  if (!strncasecmp (tline, "*FvwmCDE*DisableHints", 21))
	    tooltips_suspend ();
#endif

	  /* change the labels on the big buttons */
	  if (!strncasecmp (tline, "*FvwmCDE*Desktop1*Label", 24))
	    {
	      ch = tline + 24;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      fl_set_object_label (fd_FvwmCDE->one_button, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop2*Label", 24))
	    {
	      ch = tline + 24;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      fl_set_object_label (fd_FvwmCDE->two_button, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop3*Label", 24))
	    {
	      ch = tline + 24;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      fl_set_object_label (fd_FvwmCDE->three_button, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop4*Label", 24))
	    {
	      ch = tline + 24;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      fl_set_object_label (fd_FvwmCDE->four_button, ch);
	    }

	  /* associate a background with each desktop */
	  if (!strncasecmp (tline, "*FvwmCDE*Desktop1*Command", 25))
	    {
	      ch = tline + 25;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      strcpy (dt1back, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop2*Command", 25))
	    {
	      ch = tline + 25;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      strcpy (dt2back, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop3*Command", 25))
	    {
	      ch = tline + 25;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      strcpy (dt3back, ch);
	    }

	  if (!strncasecmp (tline, "*FvwmCDE*Desktop4*Command", 25))
	    {
	      ch = tline + 25;
	      while (isspace (*ch))
		ch++;
	      tmp = ch;
	      while (*tmp != '\0' && *tmp != '\n')
		tmp++;
	      *tmp = '\0';
	      strcpy (dt4back, ch);
	    }

	  /* window ops */
	  if (!strncasecmp (tline, "*FvwmCDE*WindowOps", 18))
	    AddFvwmCmd (tline + 18, CmdList[WINDOW_OPS]);

	  /* exit -- might not just want to quit WM */
	  if (!strncasecmp (tline, "*FvwmCDE*Exit", 13))
	    AddFvwmCmd (tline + 13, CmdList[EXIT]);

	  /* lock */
	  if (!strncasecmp (tline, "*FvwmCDE*Lock", 13))
	    AddFvwmCmd (tline + 13, CmdList[LOCK]);

	  /* taskbar background -- must be in #RRGGBB format */
	  if (!strncasecmp (tline, "*FvwmCDE*Background", 19))
	    {
	      if (!ParseColor (tline + 19, &red, &green, &blue))
		{
		  BackRed = red;
		  BackGreen = green;
		  BackBlue = blue;
		}
	    }

	  /* big button background -- also must be in given format */
	  if (!strncasecmp (tline, "*FvwmCDE*ButtonBackground", 25))
	    {
	      if (!ParseColor (tline + 25, &red, &green, &blue))
		{
		  ButtonRed = red;
		  ButtonGreen = green;
		  ButtonBlue = blue;
		}
	    }

	  /* them fancy buttons (numbered from 0..9) */
	  if (!strncasecmp (tline, "*FvwmCDE*Button0*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[0]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button1*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[1]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button2*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[2]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button3*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[3]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button4*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[4]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button5*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[5]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button6*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[6]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button7*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[7]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button8*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[8]);
	  if (!strncasecmp (tline, "*FvwmCDE*Button9*Binding", 24))
	    AddFvwmCmd (tline + 24, CmdList[9]);

	  /* hints for the biggie buttons */
#ifdef USE_HINTS
	  /* tooltips can't handle pixmaps and canvases.. */
	  if (!strncasecmp (tline, "*FvwmCDE*Button2*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button2, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button3*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button3, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button4*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button4, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button5*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button5, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button6*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button6, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button7*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button7, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button8*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button8, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Button9*Hint", 21))
	    tooltips_addtip (fd_FvwmCDE->button9, cleantip (tline + 22));
#endif

	  /* mail setup (let 'em overrule us..) */
	  if (!strncasecmp (tline, "*FvwmCDE*MailFile", 17))
	    AddFvwmCmd (tline + 17, MailFile);

	  /* the panels (ack!) numbered from 0..9 */
	  if (!strncasecmp (tline, "*FvwmCDE*Panel0*Binding", 23))
	    add_to_panel (Panel[0], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel1*Binding", 23))
	    add_to_panel (Panel[1], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel2*Binding", 23))
	    add_to_panel (Panel[2], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel3*Binding", 23))
	    add_to_panel (Panel[3], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel4*Binding", 23))
	    add_to_panel (Panel[4], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel5*Binding", 23))
	    add_to_panel (Panel[5], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel6*Binding", 23))
	    add_to_panel (Panel[6], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel7*Binding", 23))
	    add_to_panel (Panel[7], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel8*Binding", 23))
	    add_to_panel (Panel[8], tline + 23);
	  if (!strncasecmp (tline, "*FvwmCDE*Panel9*Binding", 23))
	    add_to_panel (Panel[9], tline + 23);

	  /* titles for the slideout panels */
	  if (!strncasecmp (tline, "*FvwmCDE*Panel0*Label", 21))
	    strcpy (Panel[0]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel1*Label", 21))
	    strcpy (Panel[1]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel2*Label", 21))
	    strcpy (Panel[2]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel3*Label", 21))
	    strcpy (Panel[3]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel4*Label", 21))
	    strcpy (Panel[4]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel5*Label", 21))
	    strcpy (Panel[5]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel6*Label", 21))
	    strcpy (Panel[6]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel7*Label", 21))
	    strcpy (Panel[7]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel8*Label", 21))
	    strcpy (Panel[8]->label, cleantip (tline + 22));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel9*Label", 21))
	    strcpy (Panel[9]->label, cleantip (tline + 22));

	  /* hints for the slideout panels */
#ifdef USE_HINTS
	  if (!strncasecmp (tline, "*FvwmCDE*Panel0*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[0],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel1*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[1],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel2*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[2],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel3*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[3],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel4*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[4],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel5*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[5],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel6*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[6],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel7*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[7],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel8*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[8],
			     cleantip (tline + 21));
	  if (!strncasecmp (tline, "*FvwmCDE*Panel9*Hint", 20))
	    tooltips_addtip (fd_FvwmCDE->panelbutton[9],
			     cleantip (tline + 21));
#endif
	}
    }

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: [ParseConfig] PixmapPath [%s]\n", PixmapPath);
  fprintf (stderr, "FvwmCDE: [ParseConfig]\n"
	   "Desktop1*Command [%s]\nDesktop2*Command [%s]\n"
	   "Desktop3*Command [%s]\nDesktop4*Command [%s]\n",
	   dt1back, dt2back, dt3back, dt4back);
  fflush (stderr);
#endif
  
  /* remove panel entries that weren't used */
  trim_panel (Panel[0]);
  trim_panel (Panel[1]);
  trim_panel (Panel[2]);
  trim_panel (Panel[3]);
  trim_panel (Panel[4]);
  trim_panel (Panel[5]);
  trim_panel (Panel[6]);
  trim_panel (Panel[7]);
  trim_panel (Panel[8]);
  trim_panel (Panel[9]);
}

/* handle the clock canvas */
int
canvas_expose (FL_OBJECT * ob, Window win, int w, int h, XEvent * ev, void *d)
{
  static int xc = 0, yc = 0, radius = 0;
  time_t clock_time;
  struct tm *tmval;
  int i, j, hx, hy, mx, my, angle;

  if (xc == 0)			/* initialize these coordinates */
    {
      xc = w / 2;
      yc = w / 2;
      radius = w / 2;
    }

  /* calculate the current time */
  clock_time = time (NULL);
  tmval = localtime (&clock_time);

  /* calculate coordinates for hour hand */
  angle = (tmval->tm_hour % 12) * 5 + (tmval->tm_min / 12);
  hx = XPOS (angle, 60);
  hy = YPOS (angle, 60);

  angle = tmval->tm_min;
  mx = XPOS (angle, 95);
  my = YPOS (angle, 95);

  /* fill in the background */
  XSetForeground (disp, canvasGC, fl_get_flcolor (CUSTOM_COLOR4));
  XFillRectangle (disp, win, canvasGC, 0, 0, w, h);

  /* fill in the foreground (clock hands) */
  XSetForeground (disp, canvasGC, fl_get_flcolor (FL_WHITE));
  XSetLineAttributes (disp, canvasGC, 1, LineSolid, CapProjecting,
		      JoinRound);

  /* draw the hands */
  for (i = -1; i < 2; i++)
    for (j = -1; j < 2; j++)
      {
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, hx, hy);
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, mx, my);
      }

  return (0);
}


/* handle clicks in the clock canvas (which is inside the clock button)
 * the bad part is that, clicks inside the canvas aren't reflected to the
 * clock button, so the event doesn't get handled properly..
 */
int
canvas_click (FL_OBJECT * ob, Window win, int w, int h, XEvent * ev, void *d)
{
  FD_FvwmCDE *ui = (FD_FvwmCDE *) d;

  fl_set_object_color (ui->clock_pixmap, CUSTOM_COLOR2,
		       CUSTOM_COLOR2);
  update_calendar ();
  XFlush (disp);
  SendInfoCheck (FvwmPipe, CmdList[CLOCK], 0);
  sleep (1);
  fl_set_object_color (ui->clock_pixmap, CUSTOM_COLOR1,
		       CUSTOM_COLOR1);
  update_calendar ();
  XFlush (disp);
  return (0);
}

/* update the clock without doing a full redraw (which flashes noticeably) */
void
redraw_clock (FL_OBJECT * ob)
{
  static int xc = 0, yc = 0, radius = 0;
  static int hxo = 0, hyo = 0, mxo = 0, myo = 0;
  time_t clock_time;
  struct tm *tmval;
  int i, j, hx, hy, mx, my, w, h, angle;
  Window win;
  XWindowAttributes w_attr;

  win = fl_get_canvas_id (ob);
  XGetWindowAttributes (disp, win, &w_attr);
  w = w_attr.width;
  h = w_attr.height;

  if (xc == 0)			/* initialize these coordinates */
    {
      xc = w / 2;
      yc = w / 2;
      radius = w / 2;
    }

  /* calculate the current time */
  clock_time = time (NULL);
  tmval = localtime (&clock_time);

  /* calculate coordinates for hour hand */
  angle = (tmval->tm_hour % 12) * 5 + (tmval->tm_min / 12);
  hx = XPOS (angle, 60);
  hy = YPOS (angle, 60);

  angle = tmval->tm_min;
  mx = XPOS (angle, 95);
  my = YPOS (angle, 95);

  /* fill in the background */
  XSetForeground (disp, canvasGC, fl_get_flcolor (CUSTOM_COLOR4));

  /* erase the hands */
  for (i = -1; i < 2; i++)
    for (j = -1; j < 2; j++)
      {
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, hxo, hyo);
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, mxo, myo);
      }

  /* fill in the foreground (clock hands) */
  XSetForeground (disp, canvasGC, fl_get_flcolor (FL_WHITE));
  XSetLineAttributes (disp, canvasGC, 1, LineSolid, CapProjecting,
		      JoinRound);
  /* draw the hands */
  for (i = -1; i < 2; i++)
    for (j = -1; j < 2; j++)
      {
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, hx, hy);
	XDrawLine (disp, win, canvasGC, xc + i, yc + j, mx, my);
      }
  hxo = hx;
  hyo = hy;
  mxo = mx;
  myo = my;
}

/* force-update the calendar (needed..) */
void
force_calendar (void)
{
  time_t current;
  char day[30], month[30], *ch;

  current = time (NULL);
  ch = ctime (&current);
  if (ch == NULL)
    exit (1);

  /* parse the current date */
  strcpy (month, ch + 4);
  strcpy (day, ch + 8);
  ch = strchr (month, ' ');
  if (ch != NULL)
    *ch = '\0';
  ch = strchr (day, ' ');
  if (ch != NULL)
    *ch = '\0';

  /* update screen labels */
  fl_set_object_label (fd_FvwmCDE->day_text, day);
  fl_set_object_label (fd_FvwmCDE->month_text, month);
  fl_redraw_object (fd_FvwmCDE->day_text);
  fl_redraw_object (fd_FvwmCDE->month_text);
}

/* update the calendar (and stuff..) */
void
update_calendar (void)
{
  static time_t secs = 0;
  struct stat stbuf;
  time_t current;
  char day[30], month[30], *ch;
  int nday;
  static int mailsize = 0;

  current = time (NULL);
  if ((current - secs) > UPDATE)
    {
      secs = current;
      ch = ctime (&current);
      if (ch == NULL)
	exit (1);

      /* parse the current date */
      strcpy (month, ch + 4);
      strcpy (day, ch + 7);
      ch = strchr (month, ' ');
      if (ch != NULL)
	*ch = '\0';

      nday = atoi (day);
      sprintf (day, "%d", nday);

      /* update screen labels */
      fl_set_object_label (fd_FvwmCDE->day_text, day);
      fl_set_object_label (fd_FvwmCDE->month_text, month);
      fl_redraw_object (fd_FvwmCDE->day_text);
      fl_redraw_object (fd_FvwmCDE->month_text);

      /* redraw the clock */
      redraw_clock (fd_FvwmCDE->clock_canvas);

      /* check for new mail */
      if (strlen (MailFile) > 0 && !stat (MailFile, &stbuf))
	{
	  if ((stbuf.st_size > 0) && (stbuf.st_mtime >= stbuf.st_atime))
	    {
	      /* ring the bells! */
	      if (stbuf.st_size != mailsize)
		{
		  XBell (disp, BELLVOLUME);
		  mailsize = stbuf.st_size;
		}

	      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->button4);
	      fl_set_pixmapbutton_data (fd_FvwmCDE->button4,
					mail_pixmap_xpm);
	    }
	  else
	    {
	      fl_free_pixmapbutton_pixmap (fd_FvwmCDE->button4);
	      fl_set_pixmapbutton_data (fd_FvwmCDE->button4,
					nomail_pixmap_xpm);
	    }
	}
    }
}

/* add a binding */
void
AddFvwmCmd (char *src, char *dest)
{
  char *tmp;

  while (isspace (*src))
    src++;

  strcpy (dest, src);
  tmp = strrchr (dest, '\n');
  if (tmp != NULL)
    *tmp = '\0';

#ifndef NODEBUG
  fprintf (stderr, "FvwmCDE: added binding %s\n", dest);
  fflush (stderr);
#endif
}

/* this parses a color; it MUST be in #rrggbb format */
int
ParseColor (char *col, int *r, int *g, int *b)
{
  char tmpbuf[256], *ch;
  long colnum;

  ch = col;
  memset (tmpbuf, 0, 256);
  while (*ch != '#')
    ch++;
  strcpy (tmpbuf, ch + 1);
  ch = tmpbuf;
  while (isxdigit (*ch))
    ch++;
  *ch = '\0';

  /* dumb check for validity.. */
  if (strlen (tmpbuf) != 6)
    return (-1);

  colnum = (int) strtol (tmpbuf, NULL, 16);
  *r = (int) (colnum >> 16);
  *g = (int) ((colnum >> 8) & 0xff);
  *b = (int) (colnum & 0xff);

  return (0);
}
